package tauzaman.junit;

import static org.junit.Assert.*;

import org.junit.Test;

import tauzaman.pmf.ProbabilityMassFunction;
import tauzaman.temporaldatatypes.semantics.IndeterminateSemantics;
import tauzaman.timestamp.Granule;
import tauzaman.timestamp.TimeValue;

public class TestGranule {
    @Test
    public void testCompareToDeterminate() {
        TimeValue minTV = TimeValue.BEGINNING_OF_TIME;
        TimeValue maxTV = TimeValue.END_OF_TIME;
        TimeValue oneTV = new TimeValue(1);
        TimeValue threeTV = new TimeValue(3);

        Granule min = new Granule(minTV);
        Granule max = new Granule(maxTV);
        Granule one = new Granule(oneTV);
        Granule three = new Granule(threeTV);
        
        // Is 1 < max?
        assertTrue(one.compareTo(three) < 0);
        // Is 1 < 3?
        assertTrue(one.compareTo(three) < 0);
        // Is 3 < 3?
        assertFalse(three.compareTo(three) < 0);
        // Is max < 3?
        assertFalse(max.compareTo(three) < 0);
        // Is min < 3?
        assertTrue(min.compareTo(three) < 0);
        // Is 3 < min?
        assertFalse(three.compareTo(min) < 0); 
        // Is 3 < max?
        assertTrue(three.compareTo(max) < 0);
        // Is min < min?
        assertFalse(min.compareTo(min) < 0);
        // Is max < min?
        assertFalse(max.compareTo(min) < 0);
        // Is min < max?
        assertTrue(min.compareTo(max) < 0);
        // Is max < max?
        assertFalse(max.compareTo(max) < 0);
        
        // Is 1 = max?
        assertFalse(one.compareTo(max) == 0);
        // Is 1 = 3?
        assertFalse(one.compareTo(three) == 0);
        // Is 3 = 3?
        assertTrue(three.compareTo(three) == 0);
        // Is max = 3?
        assertFalse(max.compareTo(three) == 0);
        // Is min = 3?
        assertFalse(min.compareTo(three) == 0);
        // Is 3 = min?
        assertFalse(three.compareTo(min) == 0);
        // Is 3 = max?
        assertFalse(three.compareTo(max) == 0);
        // Is min = min?
        assertTrue(min.compareTo(min) == 0);
        // Is max = min?
        assertFalse(max.compareTo(min) == 0);
        // Is min = max?
        assertFalse(min.compareTo(max) == 0);
        // Is max = max?
        assertTrue(max.compareTo(max) == 0);
        
        // Is 1 > max?
        assertFalse(one.compareTo(max) > 0);
        // Is 1 > 3?
        assertFalse(one.compareTo(three) > 0);
        // Is 3 > 1?
        assertTrue(three.compareTo(one) > 0);
        // Is 3 > 3?
        assertFalse(three.compareTo(three) > 0);
        // Is max > 3?
        assertTrue(max.compareTo(three) > 0);
        // Is min > 3?
        assertFalse(min.compareTo(three) > 0);
        // Is 3 > min?
        assertTrue(three.compareTo(min) > 0);
        // Is 3 > max?
        assertFalse(three.compareTo(max) > 0); 
        // Is min > min?
        assertFalse(min.compareTo(min) > 0);
        // Is max > min?
        assertTrue(max.compareTo(min) > 0);
        // Is min > max?
        assertFalse(min.compareTo(max) > 0);
        // Is max > max?
        assertFalse(max.compareTo(max) > 0);
    }
    @Test
    public void compareToIndeterminate() {
        TimeValue oneTV = new TimeValue(1);
        TimeValue threeTV = new TimeValue(3);
        TimeValue fourTV = new TimeValue(4);
        TimeValue fiveTV = new TimeValue(5);
        
        Granule oneToThree = new Granule(oneTV, threeTV);
        Granule oneToFour = new Granule(oneTV, fourTV);
        Granule oneToFive = new Granule(oneTV, fiveTV);
        Granule threeToFour = new Granule(threeTV, fourTV);
        Granule threeToFive = new Granule(threeTV, fiveTV);
        Granule fourToFive = new Granule(fourTV, fiveTV);
        
        // No intersection
        assertTrue(oneToThree.compareTo(threeToFive) < 0);
        assertTrue(oneToThree.compareTo(fourToFive) < 0);
        assertFalse(threeToFive.compareTo(oneToThree) < 0);
        assertFalse(fourToFive.compareTo(oneToThree) < 0);
        
        assertFalse(oneToThree.compareTo(fourToFive) == 0);
        assertFalse(fourToFive.compareTo(oneToThree) == 0);
        
        assertFalse(oneToThree.compareTo(fourToFive) > 0);
        assertTrue(fourToFive.compareTo(oneToThree) > 0);
        
        // Intersection
        assertTrue(oneToFour.compareTo(threeToFour) < 0);
        assertTrue(oneToFour.compareTo(threeToFive) < 0);
        assertTrue(oneToFive.compareTo(threeToFour) < 0);
        assertTrue(oneToFive.compareTo(threeToFive) < 0);
        
        assertFalse(oneToFour.compareTo(threeToFour) == 0);
        assertFalse(oneToFour.compareTo(threeToFive) == 0);
        assertFalse(oneToFive.compareTo(threeToFour) == 0);
        assertFalse(oneToFive.compareTo(threeToFive) == 0);
        
        assertFalse(oneToFour.compareTo(threeToFour) > 0);
        assertFalse(oneToFour.compareTo(threeToFive) > 0);
        assertFalse(oneToFive.compareTo(threeToFour) > 0);
        assertFalse(oneToFive.compareTo(threeToFive) > 0);
    }
    
    @Test
    public void testLessThan_EasyCases() {
        TimeValue oneTV = new TimeValue(1);
        TimeValue threeTV = new TimeValue(3);
        TimeValue fourTV = new TimeValue(4);
        TimeValue fiveTV = new TimeValue(5);
        
        Granule oneToThree = new Granule(oneTV, threeTV);
        Granule oneToFour = new Granule(oneTV, fourTV);
        Granule oneToFive = new Granule(oneTV, fiveTV);
        Granule threeToFour = new Granule(threeTV, fourTV);
        Granule threeToFive = new Granule(threeTV, fiveTV);
        Granule fourToFive = new Granule(fourTV, fiveTV);
        
        int definite = IndeterminateSemantics.DEFINITE;
        int possible = IndeterminateSemantics.POSSIBLE;
        
        // No Intersection
        assertTrue(oneToThree.lessThan(threeToFive, definite).isSatisfied());
        assertTrue(oneToThree.lessThan(fourToFive, definite).isSatisfied());
        assertTrue(threeToFive.lessThan(oneToThree, definite).isUnsatisfied());
        assertTrue(fourToFive.lessThan(oneToThree, definite).isUnsatisfied());
        
        assertTrue(oneToThree.lessThan(threeToFive, possible).isSatisfied());
        assertTrue(oneToThree.lessThan(fourToFive, possible).isSatisfied());
        assertTrue(threeToFive.lessThan(oneToThree, possible).isUnsatisfied());
        assertTrue(fourToFive.lessThan(oneToThree, possible).isUnsatisfied());
        
        // Definite plausibility
        assertTrue(oneToFive.lessThan(threeToFive, definite).isUnsatisfied());
        assertTrue(oneToFive.lessThan(fourToFive, definite).isUnsatisfied());
        assertTrue(oneToFive.lessThan(threeToFour, definite).isUnsatisfied());
        assertTrue(oneToFour.lessThan(threeToFive, definite).isUnsatisfied());
        
        assertTrue(threeToFive.lessThan(oneToFive, definite).isUnsatisfied());
        assertTrue(fourToFive.lessThan(oneToFive, definite).isUnsatisfied());
        assertTrue(threeToFour.lessThan(threeToFour, definite).isUnsatisfied());
        assertTrue(threeToFive.lessThan(oneToFour, definite).isUnsatisfied());
        
        // Possible plausibility
        assertTrue(oneToFive.lessThan(threeToFive, possible).isSatisfied());
        assertTrue(oneToFive.lessThan(fourToFive, possible).isSatisfied());
        assertTrue(oneToFive.lessThan(threeToFour, possible).isSatisfied());
        assertTrue(oneToFour.lessThan(threeToFive, possible).isSatisfied());
        
        assertTrue(threeToFive.lessThan(oneToFive, possible).isSatisfied());
        assertTrue(fourToFive.lessThan(oneToFive, possible).isSatisfied());
        assertTrue(threeToFour.lessThan(threeToFour, possible).isSatisfied());
        assertTrue(threeToFive.lessThan(oneToFour, possible).isSatisfied());
        
        // No pmf
        assertTrue(oneToFive.lessThan(threeToFive, 50).isMaybe());
        assertTrue(oneToFive.lessThan(fourToFive, 50).isMaybe());
        assertTrue(oneToFive.lessThan(threeToFour, 50).isMaybe());
        assertTrue(oneToFour.lessThan(threeToFive, 50).isMaybe());
        
        assertTrue(threeToFive.lessThan(oneToFive, 50).isMaybe());
        assertTrue(fourToFive.lessThan(oneToFive, 50).isMaybe());
        assertTrue(threeToFour.lessThan(threeToFour, 50).isMaybe());
        assertTrue(threeToFive.lessThan(oneToFour, 50).isMaybe());
    }
    
    @Test
    public void testLessThan_WithPMF() {
        
        float[] massfunction1 = {1, 1, 1, 1, 1, 1, 1, 1};
		float[] massfunction2 = {3, 2, 2, 1, 1, 1, 1, 1};
		ProbabilityMassFunction pmfA = ProbabilityMassFunction.getPMF(16, massfunction1);
		ProbabilityMassFunction pmfB = ProbabilityMassFunction.getPMF(64, massfunction1);
		ProbabilityMassFunction pmfC = ProbabilityMassFunction.getPMF(16, massfunction2);
		ProbabilityMassFunction pmfD = ProbabilityMassFunction.getPMF(64, massfunction2);
		/* ***** a has same range and pmf as b ***** */
		
		// Low Precision, small range
		Granule a = new Granule(1, 3, pmfA);
		Granule b = new Granule(1, 3, pmfA);
		
		assertTrue(a.lessThan(b, 50).isUnsatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isMaybe());
		assertTrue(b.lessThan(a, 30).isMaybe());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// Low Precision, medium range
		a = new Granule(1, 10, pmfA);
		b = new Granule(1, 10, pmfA);
		
		assertTrue(a.lessThan(b, 50).isMaybe());
		assertTrue(b.lessThan(a, 50).isMaybe());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// Low Precision, large range
		a = new Granule(1, 50, pmfA);
		b = new Granule(1, 50, pmfA);
		
		assertTrue(a.lessThan(b, 50).isMaybe());
		assertTrue(b.lessThan(a, 50).isMaybe());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High precision, small range
		a = new Granule(1, 3, pmfB);
		b = new Granule(1, 3, pmfB);
		
		assertTrue(a.lessThan(b, 50).isUnsatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isUnsatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High precision, medium range
		a = new Granule(1, 10, pmfB);
		b = new Granule(1, 10, pmfB);
		
		assertTrue(a.lessThan(b, 50).isUnsatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High precision, large range
		a = new Granule(1, 50, pmfB);
		b = new Granule(1, 50, pmfB);
		
		assertTrue(a.lessThan(b, 50).isMaybe());
		assertTrue(b.lessThan(a, 50).isMaybe());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		/* ****************************************** */
		
		/* ** a has different range, same pmf as b ** */
		
		// Low Precision
		a = new Granule(1, 10, pmfA);
		b = new Granule(5, 15, pmfA);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isMaybe());
		
		assertTrue(a.lessThan(b, 80).isSatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High Precision
		a = new Granule(1, 10, pmfB);
		b = new Granule(5, 15, pmfB);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isSatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// Low Precision
		a = new Granule(1, 10, pmfA);
		b = new Granule(8, 13, pmfA);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isMaybe());
		
		assertTrue(a.lessThan(b, 80).isSatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isMaybe());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High Precision
		a = new Granule(1, 10, pmfB);
		b = new Granule(8, 13, pmfB);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 80).isSatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		/* ******************************************** */
		
		/* ** a has same range, different pmf from b ** */
		// Low precision, small range
		a = new Granule(1, 3, pmfC);
		b = new Granule(1, 3, pmfA);
		
		assertTrue(a.lessThan(b, 50).isUnsatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// Low precision, medium range
		a = new Granule(1, 10, pmfC);
		b = new Granule(1, 10, pmfA);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isMaybe());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// Low precision, large range
		a = new Granule(1, 50, pmfC);
		b = new Granule(1, 50, pmfA);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High precision, small range
		a = new Granule(1, 3, pmfD);
		b = new Granule(1, 3, pmfB);
		
		assertTrue(a.lessThan(b, 50).isUnsatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High precision, medium range
		a = new Granule(1, 10, pmfD);
		b = new Granule(1, 10, pmfB);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		// High precision, large range
		a = new Granule(1, 50, pmfD);
		b = new Granule(1, 50, pmfB);
		
		assertTrue(a.lessThan(b, 50).isSatisfied());
		assertTrue(b.lessThan(a, 50).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 30).isSatisfied());
		assertTrue(b.lessThan(a, 30).isSatisfied());
		
		assertTrue(a.lessThan(b, 10).isSatisfied());
		assertTrue(b.lessThan(a, 10).isSatisfied());
		
		assertTrue(a.lessThan(b, 80).isUnsatisfied());
		assertTrue(b.lessThan(a, 80).isUnsatisfied());
		
		assertTrue(a.lessThan(b, 95).isUnsatisfied());
		assertTrue(b.lessThan(a, 95).isUnsatisfied());
		
		/* ******************************************** */
    }
    
    @Test
    public void testEqualTo_EasyCases() {
        TimeValue oneTV = new TimeValue(1);
        TimeValue threeTV = new TimeValue(3);
        TimeValue fourTV = new TimeValue(4);
        TimeValue fiveTV = new TimeValue(5);
        
        Granule oneToThree = new Granule(oneTV, threeTV);
        Granule oneToFour = new Granule(oneTV, fourTV);
        Granule oneToFive = new Granule(oneTV, fiveTV);
        Granule threeToFour = new Granule(threeTV, fourTV);
        Granule threeToFive = new Granule(threeTV, fiveTV);
        Granule fourToFive = new Granule(fourTV, fiveTV);
        
        int definite = IndeterminateSemantics.DEFINITE;
        int possible = IndeterminateSemantics.POSSIBLE;
        
        // No Intersection
        assertTrue(oneToThree.equalTo(fourToFive, definite).isUnsatisfied());
        assertTrue(fourToFive.equalTo(oneToThree, definite).isUnsatisfied());
        
        assertTrue(oneToThree.equalTo(fourToFive, possible).isUnsatisfied());
        assertTrue(fourToFive.equalTo(oneToThree, possible).isUnsatisfied());
        
        // Definite plausibility
        assertTrue(oneToFive.equalTo(threeToFive, definite).isUnsatisfied());
        assertTrue(oneToFive.equalTo(fourToFive, definite).isUnsatisfied());
        assertTrue(oneToFive.equalTo(threeToFour, definite).isUnsatisfied());
        assertTrue(oneToFour.equalTo(threeToFive, definite).isUnsatisfied());
        
        assertTrue(threeToFive.equalTo(oneToFive, definite).isUnsatisfied());
        assertTrue(fourToFive.equalTo(oneToFive, definite).isUnsatisfied());
        assertTrue(threeToFour.equalTo(threeToFour, definite).isUnsatisfied());
        assertTrue(threeToFive.equalTo(oneToFour, definite).isUnsatisfied());
        
        // Possible plausibility
        assertTrue(oneToFive.equalTo(threeToFive, possible).isSatisfied());
        assertTrue(oneToFive.equalTo(fourToFive, possible).isSatisfied());
        assertTrue(oneToFive.equalTo(threeToFour, possible).isSatisfied());
        assertTrue(oneToFour.equalTo(threeToFive, possible).isSatisfied());
        
        assertTrue(threeToFive.equalTo(oneToFive, possible).isSatisfied());
        assertTrue(fourToFive.equalTo(oneToFive, possible).isSatisfied());
        assertTrue(threeToFour.equalTo(threeToFour, possible).isSatisfied());
        assertTrue(threeToFive.equalTo(oneToFour, possible).isSatisfied());
        
        // No pmf
        assertTrue(oneToFive.equalTo(threeToFive, 50).isMaybe());
        assertTrue(oneToFive.equalTo(fourToFive, 50).isMaybe());
        assertTrue(oneToFive.equalTo(threeToFour, 50).isMaybe());
        assertTrue(oneToFour.equalTo(threeToFive, 50).isMaybe());
        
        assertTrue(threeToFive.equalTo(oneToFive, 50).isMaybe());
        assertTrue(fourToFive.equalTo(oneToFive, 50).isMaybe());
        assertTrue(threeToFour.equalTo(threeToFour, 50).isMaybe());
        assertTrue(threeToFive.equalTo(oneToFour, 50).isMaybe());
    }
    
    @Test
    public void testEqualTo_WithPMF() {
        
        float[] massfunction1 = {1, 1, 1, 1, 1, 1, 1, 1};
		float[] massfunction2 = {3, 2, 2, 1, 1, 1, 1, 1};
		ProbabilityMassFunction pmfA = ProbabilityMassFunction.getPMF(16, massfunction1);
		ProbabilityMassFunction pmfB = ProbabilityMassFunction.getPMF(64, massfunction1);
		ProbabilityMassFunction pmfC = ProbabilityMassFunction.getPMF(16, massfunction2);
		ProbabilityMassFunction pmfD = ProbabilityMassFunction.getPMF(64, massfunction2);
		/* ***** a has same range and pmf as b ***** */
		
		// Low Precision, small range
		Granule a = new Granule(1, 3, pmfA);
		Granule b = new Granule(1, 3, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isSatisfied());
		assertTrue(b.equalTo(a, 10).isSatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// Low Precision, medium range
		a = new Granule(1, 10, pmfA);
		b = new Granule(1, 10, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isMaybe());
		assertTrue(b.equalTo(a, 10).isMaybe());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// Low Precision, large range
		a = new Granule(1, 50, pmfA);
		b = new Granule(1, 50, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// High precision, small range
		a = new Granule(1, 3, pmfB);
		b = new Granule(1, 3, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 2).isSatisfied());
		assertTrue(b.equalTo(a, 2).isSatisfied());
		
		// High precision, medium range
		a = new Granule(1, 10, pmfB);
		b = new Granule(1, 10, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 2).isSatisfied());
		assertTrue(b.equalTo(a, 2).isSatisfied());
		
		// High precision, large range
		a = new Granule(1, 50, pmfB);
		b = new Granule(1, 50, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 2).isMaybe());
		assertTrue(b.equalTo(a, 2).isMaybe());
		
		/* ****************************************** */
		
		/* ** a has different range, same pmf as b ** */
		
		// Low Precision
		a = new Granule(1, 10, pmfA);
		b = new Granule(5, 15, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// High Precision
		a = new Granule(1, 10, pmfB);
		b = new Granule(5, 15, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// Low Precision
		a = new Granule(1, 10, pmfA);
		b = new Granule(3, 8, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isMaybe());
		assertTrue(b.equalTo(a, 10).isMaybe());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// High Precision
		a = new Granule(1, 10, pmfB);
		b = new Granule(3, 8, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		/* ******************************************** */
		
		/* ** a has same range, different pmf from b ** */
		// Low precision, small range
		a = new Granule(1, 3, pmfC);
		b = new Granule(1, 3, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isSatisfied());
		assertTrue(b.equalTo(a, 10).isSatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// Low precision, medium range
		a = new Granule(1, 10, pmfC);
		b = new Granule(1, 10, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isMaybe());
		assertTrue(b.equalTo(a, 10).isMaybe());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// Low precision, large range
		a = new Granule(1, 50, pmfC);
		b = new Granule(1, 50, pmfA);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		// High precision, small range
		a = new Granule(1, 3, pmfD);
		b = new Granule(1, 3, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 2).isSatisfied());
		assertTrue(b.equalTo(a, 2).isSatisfied());
		
		// High precision, medium range
		a = new Granule(1, 10, pmfD);
		b = new Granule(1, 10, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 2).isSatisfied());
		assertTrue(b.equalTo(a, 2).isSatisfied());
		
		// High precision, large range
		a = new Granule(1, 50, pmfD);
		b = new Granule(1, 50, pmfB);
		
		assertTrue(a.equalTo(b, 50).isUnsatisfied());
		assertTrue(b.equalTo(a, 50).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 30).isUnsatisfied());
		assertTrue(b.equalTo(a, 30).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 10).isUnsatisfied());
		assertTrue(b.equalTo(a, 10).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 80).isUnsatisfied());
		assertTrue(b.equalTo(a, 80).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 95).isUnsatisfied());
		assertTrue(b.equalTo(a, 95).isUnsatisfied());
		
		assertTrue(a.equalTo(b, 2).isMaybe());
		assertTrue(b.equalTo(a, 2).isMaybe());
		
		/* ******************************************** */
    }
    
}
