package tauzaman.temporaldatatypes.semantics;

import tauzaman.calendricsystem.CalendricSystemServiceException;
import tauzaman.temporaldatatypes.Instant;
import tauzaman.temporaldatatypes.Interval;
import tauzaman.temporaldatatypes.Period;

/**
 * The <code>LeftOperandIndeterminateSemantics</code> class performs arithmetic
 * and comparison operations on indeterminate time values. Given two time 
 * values on an operation, the <code>LeftOperandIndeterminateSemantics</code>
 * converts the right time value to the left time value's granularity. If the right
 * argument cannot be converted to the left, an error is thrown.
 * 
 * @author  Alex Henniges and Richard Snodgrass
 * @version 2.0, Sep/2/2008
 * @see     tauzaman.temporaldatatypes.Instant
 * @see     tauzaman.temporaldatatypes.semantics.IndeterminateSemantics
 */
public class LeftOperandIndeterminateSemantics extends IndeterminateSemantics{

	public void scalePair(Instant left, Instant right) 
			throws CalendricSystemServiceException {
		this.leftInstant = left;
		this.rightInstant = right.scale(left.getGranularity());
	}

	public void scalePair(Period left, Period right) 
			throws CalendricSystemServiceException {
		this.leftPeriod = left;
		this.rightPeriod = right.scale(left.getGranularity());
	}
	
	public void scalePair(Period left, Interval right)
			throws CalendricSystemServiceException {
		this.leftPeriod = left;
		this.rightInterval = right.scale(left.getGranularity());
	}

	public void scalePair(Interval left, Interval right) 
			throws CalendricSystemServiceException {
		this.leftInterval = left;
		this.rightInterval = right.scale(left.getGranularity());
	}

	public void scalePair(Instant left, Interval right) 
			throws CalendricSystemServiceException {
		this.leftInstant = left;
		this.rightInterval = right.scale(left.getGranularity());
	}
}
