package tauzaman.temporaldatatypes.semantics;

import tauzaman.calendricsystem.CalendricSystemServiceException;
import tauzaman.temporaldatatypes.Instant;
import tauzaman.temporaldatatypes.Interval;
import tauzaman.temporaldatatypes.Period;

/**
 * The <code>RightOperandIndeterminateSemantics</code> class performs arithmetic
 * and comparison operations on indeterminate time values. Given two time 
 * values on an operation, the <code>RightOperandIndeterminateSemantics</code>
 * converts the left time value to the right time value's granularity. If the left
 * argument cannot be converted to the right, an error is thrown.
 * 
 * @author  Alex Henniges and Richard Snodgrass
 * @version 2.0, Sep/2/2008
 * @see     tauzaman.temporaldatatypes.Instant
 * @see     tauzaman.temporaldatatypes.semantics.IndeterminateSemantics
 */
public class RightOperandIndeterminateSemantics extends IndeterminateSemantics {
	
	public void scalePair(Instant left, Instant right) 
			throws CalendricSystemServiceException {
		this.leftInstant = left.scale(right.getGranularity());
		this.rightInstant = right;
	}

	public void scalePair(Period left, Period right) 
			throws CalendricSystemServiceException {
		this.leftPeriod = left.scale(right.getGranularity());
		this.rightPeriod = right;
	}

	public void scalePair(Period left, Interval right)
			throws CalendricSystemServiceException {
		this.leftPeriod = left.scale(right.getGranularity());
		this.rightInterval = right;
	}

	public void scalePair(Interval left, Interval right) 
			throws CalendricSystemServiceException {
		this.leftInterval = left.scale(right.getGranularity());
		this.rightInterval = right;
	}

	public void scalePair(Instant left, Interval right) 
			throws CalendricSystemServiceException {
		this.leftInstant = left.scale(right.getGranularity());
		this.rightInterval = right;
	}
}
