package tauzaman.tests.junit;
import static org.junit.Assert.*;

import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.junit.Before;
import org.junit.Test;
import org.w3c.dom.Element;

import tauzaman.XMLParseException;
import tauzaman.XMLParser;
import tauzaman.property.Property;
import tauzaman.property.PropertyFormationException;
import tauzaman.property.PropertyStackService;


public class PropertyStackServiceTest {
    
    private static PropertyStackService pss;
    private static URL testURL;
    private static XMLParser parser;
    private static Hashtable<String, URL> bogusMapper;
/*    
    private static String[] propertyNames = { "Locale", "InstantInputFormat",
            "InstantOutputFormat", "IntervalInputFormat",
            "IntervalOutputFormat", "NowRelativeInstantInputFormat",
            "NowRelativeInstantOutputFormat", "PeriodInputFormat",
            "PeriodOutputFormat", "IndeterminateInstantInputFormat",
            "IndeterminateInstantOutputFormat", "OverrideInputOrder" };
*/
    @Before
    public void setUp() throws Exception {
        parser = new XMLParser();
        pss = new PropertyStackService();
        testURL = new URL("file:/" + System.getProperty("user.dir")
                + "/src/tauzaman/tests/iotests/files/propertyTable1.xml");
        bogusMapper = new Hashtable<String, URL>();
    }

    @Test
    public final void testPushPeekandPeekAll()
            throws PropertyFormationException, XMLParseException {
        
        // testing push
        Element root = parser.parseURL(testURL);

        // only testing a selection of the propery names
        String[] someNames = { "Locale", "IntervalOutputFormat",
                "PeriodInputFormat", "OverrideInputOrder" };
        Element[] someElements = new Element[4];
        Property[] someProperties = new Property[4];

        for (int i = 0; i < 4; i++) {
            String leftName = someNames[i];
            boolean found = false;
            Vector<Element> propertyNodes = parser.locator(root, "property");
            for (int j = 0; j < propertyNodes.size(); j++) {
                Element propertyNode = propertyNodes.elementAt(j);
                String propertyName = propertyNode.getAttribute("name");
                if (propertyName.equals(leftName)) {
                    Property property = new Property(propertyName, testURL,
                            propertyNode, bogusMapper);
                    pss.push(property);
                    someProperties[i] = property;
                    someElements[i] = propertyNode;
                    found = true;
                    break;
                }
            }
            if (!found) {
                System.out.println("name " + someNames[i] + " not found");
            }
        }

        for (int i = 0; i < 4; i++) {
            Property prop = pss.peek(someNames[i]);
            assertTrue(prop == someProperties[i]);
        }
        for (int i = 0; i < 4; i++) {
            Enumeration<Property> props = pss.peekAll(someNames[i]);
            assertTrue(props.nextElement() == someProperties[i]);
        }
    }
}
